@@ -18,13 +18,13 @@ fn check_in_deleted_directory_errors() {
     set_current_dir(&temp_path).unwrap();
     drop(temp_dir);
 
-    assert_cmd_snapshot!(Command::new(get_cargo_bin(BIN_NAME)).arg("check"), @r###"
-            success: false
-            exit_code: 2
-            ----- stdout -----
+    assert_cmd_snapshot!(Command::new(get_cargo_bin(BIN_NAME)).arg("check"), @"
+    success: false
+    exit_code: 2
+    ----- stdout -----
 
-            ----- stderr -----
-            ruff failed
-              Cause: Working directory does not exist
-            "###);
+    ----- stderr -----
+    ruff failed
+      Cause: Working directory does not exist
+    ");
 }
