@@ -11,7 +11,7 @@ use crate::checkers::ast::Checker;
 
 /// ## What it does
 /// Checks for `except` clauses that catch all exceptions.  This includes
-/// bare `except`, `except BaseException` and `except Exception`.
+/// `except BaseException` and `except Exception`.
 ///
 ///
 /// ## Why is this bad?
@@ -58,11 +58,16 @@ use crate::checkers::ast::Checker;
 ///     logging.exception("Something went wrong")
 /// ```
 ///
+/// ## Options
+///
+/// - `lint.logger-objects`
+///
 /// ## References
 /// - [Python documentation: The `try` statement](https://docs.python.org/3/reference/compound_stmts.html#the-try-statement)
 /// - [Python documentation: Exception hierarchy](https://docs.python.org/3/library/exceptions.html#exception-hierarchy)
 /// - [PEP 8: Programming Recommendations on bare `except`](https://peps.python.org/pep-0008/#programming-recommendations)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.127")]
 pub(crate) struct BlindExcept {
     name: String,
 }
@@ -149,24 +154,28 @@ impl<'a> ReraiseVisitor<'a> {
 
 impl<'a> StatementVisitor<'a> for ReraiseVisitor<'a> {
     fn visit_stmt(&mut self, stmt: &'a Stmt) {
+        if self.seen {
+            return;
+        }
         match stmt {
             Stmt::Raise(ast::StmtRaise { exc, cause, .. }) => {
-                if let Some(cause) = cause {
-                    if let Expr::Name(ast::ExprName { id, .. }) = cause.as_ref() {
-                        if self.name.is_some_and(|name| id == name) {
-                            self.seen = true;
-                        }
+                // except Exception [as <name>]:
+                //     raise [<exc> [from <cause>]]
+                let reraised = match (self.name, exc.as_deref(), cause.as_deref()) {
+                    // `raise`
+                    (_, None, None) => true,
+                    // `raise SomeExc from <name>`
+                    (Some(name), _, Some(Expr::Name(ast::ExprName { id, .. }))) if name == id => {
+                        true
                     }
-                } else {
-                    if let Some(exc) = exc {
-                        if let Expr::Name(ast::ExprName { id, .. }) = exc.as_ref() {
-                            if self.name.is_some_and(|name| id == name) {
-                                self.seen = true;
-                            }
-                        }
-                    } else {
-                        self.seen = true;
+                    // `raise <name>` and `raise <name> from SomeCause`
+                    (Some(name), Some(Expr::Name(ast::ExprName { id, .. })), _) if name == id => {
+                        true
                     }
+                    _ => false,
+                };
+                if reraised {
+                    self.seen = true;
                 }
             }
             Stmt::Try(_) | Stmt::FunctionDef(_) | Stmt::ClassDef(_) => {}
@@ -200,6 +209,9 @@ impl<'a> LogExceptionVisitor<'a> {
 
 impl<'a> StatementVisitor<'a> for LogExceptionVisitor<'a> {
     fn visit_stmt(&mut self, stmt: &'a Stmt) {
+        if self.seen {
+            return;
+        }
         match stmt {
             Stmt::Expr(ast::StmtExpr { value, .. }) => {
                 if let Expr::Call(ast::ExprCall {
