@@ -32,9 +32,14 @@ use crate::{Edit, Fix, FixAvailability, Violation};
 ///     bar(i)
 /// ```
 ///
+/// ## Options
+///
+/// - `lint.dummy-variable-rgx`
+///
 /// ## References
 /// - [PEP 8: Naming Conventions](https://peps.python.org/pep-0008/#naming-conventions)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.84")]
 pub(crate) struct UnusedLoopControlVariable {
     /// The name of the loop control variable.
     name: String,
