@@ -1,9 +1,10 @@
 use ruff_python_ast::Identifier;
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
+use crate::checkers::ast::Checker;
 use crate::rules::pycodestyle::helpers::is_ambiguous_name;
 
 /// ## What it does
@@ -14,18 +15,19 @@ use crate::rules::pycodestyle::helpers::is_ambiguous_name;
 /// numerals one and zero. When tempted to use 'l', use 'L' instead.
 ///
 /// ## Example
+///
 /// ```python
-/// class I(object):
-///     ...
+/// class I(object): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
-/// class Integer(object):
-///     ...
+/// class Integer(object): ...
 /// ```
-#[violation]
-pub struct AmbiguousClassName(pub String);
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.35")]
+pub(crate) struct AmbiguousClassName(pub String);
 
 impl Violation for AmbiguousClassName {
     #[derive_message_formats]
@@ -36,13 +38,8 @@ impl Violation for AmbiguousClassName {
 }
 
 /// E742
-pub(crate) fn ambiguous_class_name(name: &Identifier) -> Option<Diagnostic> {
+pub(crate) fn ambiguous_class_name(checker: &Checker, name: &Identifier) {
     if is_ambiguous_name(name) {
-        Some(Diagnostic::new(
-            AmbiguousClassName(name.to_string()),
-            name.range(),
-        ))
-    } else {
-        None
+        checker.report_diagnostic(AmbiguousClassName(name.to_string()), name.range());
     }
 }
