@@ -1,5 +1,6 @@
 use anyhow::Result;
-use lsp_types::{Position, notification::ShowMessage, request::RegisterCapability};
+use lsp_types::notification::ShowMessage;
+use lsp_types::{Position, request::RegisterCapability};
 use ruff_db::system::SystemPath;
 use serde_json::Value;
 use ty_server::{ClientOptions, DiagnosticMode};
@@ -294,7 +295,7 @@ def foo() -> str:
         .build()
         .wait_until_workspaces_are_initialized();
 
-    server.open_text_document(foo, &foo_content, 1);
+    server.open_text_document(foo, foo_content, 1);
     let hover = server.hover_request(foo, Position::new(0, 5));
 
     assert!(
@@ -326,7 +327,7 @@ def foo() -> str:
         .build()
         .wait_until_workspaces_are_initialized();
 
-    server.open_text_document(foo, &foo_content, 1);
+    server.open_text_document(foo, foo_content, 1);
     let hover = server.hover_request(foo, Position::new(0, 5));
 
     assert!(
@@ -367,14 +368,14 @@ def bar() -> str:
         .build()
         .wait_until_workspaces_are_initialized();
 
-    server.open_text_document(foo, &foo_content, 1);
+    server.open_text_document(foo, foo_content, 1);
     let hover_foo = server.hover_request(foo, Position::new(0, 5));
     assert!(
         hover_foo.is_none(),
         "Expected no hover information for workspace A, got: {hover_foo:?}"
     );
 
-    server.open_text_document(bar, &bar_content, 1);
+    server.open_text_document(bar, bar_content, 1);
     let hover_bar = server.hover_request(bar, Position::new(0, 5));
     assert!(
         hover_bar.is_some(),
@@ -402,7 +403,7 @@ fn unknown_initialization_options() -> Result<()> {
     insta::assert_json_snapshot!(show_message_params, @r#"
     {
       "type": 2,
-      "message": "Received unknown options during initialization: 'bar'. Refer to the logs for more details"
+      "message": "Received unknown options during initialization: {\n  /"bar/": null\n}"
     }
     "#);
 
@@ -427,103 +428,28 @@ fn unknown_options_in_workspace_configuration() -> Result<()> {
     insta::assert_json_snapshot!(show_message_params, @r#"
     {
       "type": 2,
-      "message": "Received unknown options for workspace `file://<temp_dir>/foo`: 'bar'. Refer to the logs for more details."
+      "message": "Received unknown options for workspace `file://<temp_dir>/foo`: {\n  /"bar/": null\n}"
     }
     "#);
 
     Ok(())
 }
 
-/// Tests that the server sends a registration request for the rename capability if the client
-/// setting is set to true and dynamic registration is enabled.
-#[test]
-fn register_rename_capability_when_enabled() -> Result<()> {
-    let workspace_root = SystemPath::new("foo");
-    let mut server = TestServerBuilder::new()?
-        .with_workspace(workspace_root, None)?
-        .with_initialization_options(ClientOptions::default().with_experimental_rename(true))
-        .enable_rename_dynamic_registration(true)
-        .build()
-        .wait_until_workspaces_are_initialized();
-
-    let (_, params) = server.await_request::<RegisterCapability>();
-    let [registration] = params.registrations.as_slice() else {
-        panic!(
-            "Expected a single registration, got: {:#?}",
-            params.registrations
-        );
-    };
-
-    insta::assert_json_snapshot!(registration, @r#"
-    {
-      "id": "ty/textDocument/rename",
-      "method": "textDocument/rename",
-      "registerOptions": {
-        "prepareProvider": true
-      }
-    }
-    "#);
-
-    Ok(())
-}
-
-/// Tests that rename capability is statically registered during initialization if the client
-/// doesn't support dynamic registration, but the server is configured to support it.
-#[test]
-fn rename_available_without_dynamic_registration() -> Result<()> {
-    let workspace_root = SystemPath::new("foo");
-
-    let server = TestServerBuilder::new()?
-        .with_workspace(workspace_root, None)?
-        .with_initialization_options(ClientOptions::default().with_experimental_rename(true))
-        .enable_rename_dynamic_registration(false)
-        .build()
-        .wait_until_workspaces_are_initialized();
-
-    let initialization_result = server.initialization_result().unwrap();
-    insta::assert_json_snapshot!(initialization_result.capabilities.rename_provider, @r#"
-    {
-      "prepareProvider": true
-    }
-    "#);
-
-    Ok(())
-}
-
-/// Tests that the server does not send a registration request for the rename capability if the
-/// client setting is set to false and dynamic registration is enabled.
-#[test]
-fn not_register_rename_capability_when_disabled() -> Result<()> {
-    let workspace_root = SystemPath::new("foo");
-
-    TestServerBuilder::new()?
-        .with_workspace(workspace_root, None)?
-        .with_initialization_options(ClientOptions::default().with_experimental_rename(false))
-        .enable_rename_dynamic_registration(true)
-        .build()
-        .wait_until_workspaces_are_initialized();
-
-    // The `Drop` implementation will make sure that the client did not receive any registration
-    // request.
-
-    Ok(())
-}
-
 /// Tests that the server can register multiple capabilities at once.
 ///
 /// This test would need to be updated when the server supports additional capabilities in the
 /// future.
+///
+/// TODO: This test currently only verifies a single capability. It should be
+/// updated with more dynamic capabilities when the server supports it.
 #[test]
 fn register_multiple_capabilities() -> Result<()> {
     let workspace_root = SystemPath::new("foo");
     let mut server = TestServerBuilder::new()?
         .with_workspace(workspace_root, None)?
         .with_initialization_options(
-            ClientOptions::default()
-                .with_experimental_rename(true)
-                .with_diagnostic_mode(DiagnosticMode::Workspace),
+            ClientOptions::default().with_diagnostic_mode(DiagnosticMode::Workspace),
         )
-        .enable_rename_dynamic_registration(true)
         .enable_diagnostic_dynamic_registration(true)
         .build()
         .wait_until_workspaces_are_initialized();
@@ -531,8 +457,6 @@ fn register_multiple_capabilities() -> Result<()> {
     let (_, params) = server.await_request::<RegisterCapability>();
     let registrations = params.registrations;
 
-    assert_eq!(registrations.len(), 2);
-
     insta::assert_json_snapshot!(registrations, @r#"
     [
       {
@@ -545,16 +469,33 @@ fn register_multiple_capabilities() -> Result<()> {
           "workDoneProgress": true,
           "workspaceDiagnostics": true
         }
-      },
-      {
-        "id": "ty/textDocument/rename",
-        "method": "textDocument/rename",
-        "registerOptions": {
-          "prepareProvider": true
-        }
       }
     ]
     "#);
 
     Ok(())
 }
+
+/// Tests that the server doesn't panic when `VIRTUAL_ENV` points to a non-existent directory.
+///
+/// See: <https://github.com/astral-sh/ty/issues/2031>
+#[test]
+fn missing_virtual_env_does_not_panic() -> Result<()> {
+    let workspace_root = SystemPath::new("project");
+
+    // This should not panic even though VIRTUAL_ENV points to a non-existent path
+    let mut server = TestServerBuilder::new()?
+        .with_workspace(workspace_root, None)?
+        .with_env_var("VIRTUAL_ENV", "/nonexistent/virtual/env/path")
+        .build()
+        .wait_until_workspaces_are_initialized();
+
+    let _show_message_params = server.await_notification::<ShowMessage>();
+
+    // Something accursed in the escaping pipeline produces `\/` in windows paths
+    // and I can't for the life of me get insta to escape it properly, so I just
+    // need to move on with my life and not debug this right now, but ideally we
+    // would snapshot the message here.
+
+    Ok(())
+}
